package org.obolibrary.oboformat.model;

import java.util.HashSet;
import java.util.Set;

import org.obolibrary.oboformat.parser.OBOFormatConstants.OboFormatTag;

/**
 * See "sublanguages", section 6 of spec
 * 
 * @author cjm
 *
 */
public class OBOProfileUtil {

	public enum ProfileType { DAG, NO_DANGLING, UNIDRECTIONAL, FULLY_ASSERTED, FULL_LABELED, NO_EQUIVALENCIES, NO_NAMESPACE_CROSSING,
		NO_QUALIFIERS,
		NO_PROPERTY_VALUES, // not yet in spec
		BASIC
	}

	public class UnrepairableProfileViolation extends Exception {

		public UnrepairableProfileViolation(ProfileType pt,
				Frame frame) {
			super("Profile error: "+pt+" in "+frame.toString());
		}

	}

	/**
	 * @param doc
	 * @return
	 * @throws UnrepairableProfileViolation 
	 */
	public boolean enforce(OBODoc doc, Set<ProfileType> profiles) throws UnrepairableProfileViolation {
		Set<Frame> modifiedFrames = new HashSet<Frame>();
		for (Frame frame : getAllFrames(doc)) {
			Set<Clause> rmClauses = new HashSet<Clause>();

			if (isProfile(profiles, ProfileType.FULL_LABELED)) {
				if (frame.equals(doc.getHeaderFrame())) {
					// no check
				}
				else {
					Clause c = frame.getClause(OboFormatTag.TAG_NAME);
					if (c == null || c.getValue().equals("")) {
						throw new UnrepairableProfileViolation(ProfileType.FULL_LABELED, frame);
					}
				}
			}


			for (Clause c : frame.getClauses()) {

				if (isProfile(profiles, ProfileType.DAG)) {
					// TODO
				}

				if (isProfile(profiles, ProfileType.NO_DANGLING)) {
					// TODO 
				}

				if (isProfile(profiles, ProfileType.UNIDRECTIONAL)) {
					// TODO - omntology specific
				}

				if (isProfile(profiles, ProfileType.FULLY_ASSERTED)) {
					// TODO - requires reasoner
				}


				if (isProfile(profiles, ProfileType.NO_QUALIFIERS)) {
					if (c.getQualifierValues().size() > 0) {
						c.setQualifierValues(new HashSet<QualifierValue>());
						modifiedFrames.add(frame);
					}
				}

				if (isProfile(profiles, ProfileType.NO_EQUIVALENCIES)) {
					if (c.getTag().equals(OboFormatTag.TAG_EQUIVALENT_TO) ||
							c.getTag().equals(OboFormatTag.TAG_INTERSECTION_OF) ||
							c.getTag().equals(OboFormatTag.TAG_UNION_OF) ||
							c.getTag().equals(OboFormatTag.TAG_EQUIVALENT_TO_CHAIN)) {
						rmClauses.add(c);
						modifiedFrames.add(frame);
					}
				}

				if (isProfile(profiles, ProfileType.NO_PROPERTY_VALUES)) {
					if (c.getTag().equals(OboFormatTag.TAG_PROPERTY_VALUE)) {
						rmClauses.add(c);
						modifiedFrames.add(frame);
					}
				}


			}
			frame.clauses.removeAll(rmClauses);
		}
		return true;
	}

	private boolean isProfile(Set<ProfileType> profiles, ProfileType t) {
		if (profiles.contains(ProfileType.BASIC)) {
			return true;
		}
		
		if (profiles.contains(t)) {
			return true;
		}
		return false;
	}

	private Set<Frame> getAllFrames(OBODoc doc) {
		Set<Frame> fs = new HashSet<Frame>();
		fs.add(doc.getHeaderFrame());
		fs.addAll(doc.getTermFrames());
		fs.addAll(doc.getTypedefFrames());
		fs.addAll(doc.getInstanceFrames());
		return fs;
	}

}
